#include "tp2.h"

/**********************************
 * Linear Feedback Shift Register
 **********************************/

// global variables: state for galois
static unsigned long galois_state = 1;        // galois state
static unsigned long galois_taps = 0;         // galois taps, given by the "1" bits in galois_taps
static int galois_size = 0;                   // state size, in bits

// initialize the current galois generator
void galois_init(unsigned long taps) {
    galois_taps = taps;
    galois_size = 0;
    for (int i=0;i<64;i++) {
        if (BIT(i,taps) == 1) {
            galois_size = i+1;
        }
    }
}

// give the initial seed for the current galois generator
// NOTE: do not use a seed of 0!
void galois_seed(unsigned long seed) {
    galois_state = seed;
}

// generate the next bit
int galois_random_bit() {
    int bit = galois_state & 1;
    galois_state = galois_state>>1;
    if (bit) galois_state ^= galois_taps;
    return bit;
}

// display the current galois
void galois_show() {
    printf("\n");
    printf("b_n = ");
    int first = 1;
    for (int i=0;i<64;i++) {
        if (BIT(i,galois_taps) == 1) {
            if (first == 0) printf("+ ");
            printf("b_n-%d ", i+1);
            first = 0;
        }
    }
    printf("(taps: "); print_bin(galois_taps,galois_size); printf(")\n");
    printf("\n");
    /* printf("current state: 0x%lx (", galois_state); */
    /* print_bin(galois_state,galois_size);printf(")\n"); */
    /* printf("\n"); */
}

// look for the period of the current galois generator with given seed
// TODO
int galois_period(unsigned long seed);
