/*
 * info-528,
 * entêtes pour le tp2
 */

#ifndef _TP_PRNG_H
#define _TP_PRNG_H

#include <stdlib.h>
#include <stdio.h>
#include <getopt.h>
#include <assert.h>
#include <string.h>

#define BIT(i,m)            (((m)>>(i)) & 1)


/**********************************
 * quelques fonctions utilitaires
 **********************************/

// affiche un entier long en binaire
void print_bin(unsigned long m, int t);

// calcule le pgcd et les nombres de Bezout pour a et b
void pgcd_bezout (long* g, long* x, long* y, long a, long b);

// calcul le pgcd de a et b
long pgcd(long a, long b);


/*********************************
 * Linear congruential generator
 *********************************/

// initialize the current LCG
void LCG_init(long unsigned a, long unsigned c, long unsigned m);

// give the initial seed for the current LCG
void LCG_seed(long unsigned seed);

// generate the next number
long unsigned LCG_random() ;

// display the current LCG
void LCG_show();


/**********************************
 * Linear Feedback Shift Register
 **********************************/

// initialize the current LFSR
void LFSR_init(unsigned long taps);

// give the initial seed for the current LFSR
void LFSR_seed(unsigned long seed);

// generate the next bit
int LFSR_random_bit();

// display the current LFSR
void LFSR_show();


/*****************************************
 * Galois Linear Feedback Shift Register
 *****************************************/

// initialize the current galois generator
void galois_init(unsigned long taps);

// give the initial seed for the current galois generator
void galois_seed(unsigned long seed);

// generate the next bit
int galois_random_bit();

// display the current galois
void galois_show();


/**********************************
 * À FAIRE
 **********************************/

/*
 * calcule l'inverse de a modulo m
 * (renvoie 0 si a n'est pas premier avec m)
 */
long invert_mod(long a, long m);


/*
 * craque un générateur congruentiel en cherchant le module, puis les nombres
 * a et c à partir de nombre générés
 *
 * Le tableau 'random' contient les 'nb' premiers nombres générés.
 * La fonction renvoie 1 si le générateur trouvé génère bien la liste donnée et -1 sinon.
 */
int LCG_crack(int nb, long unsigned *random, long unsigned* a, long unsigned* c, long unsigned* m);


/*
 * diagonalise une matrice booléenne de taille n x n+1
 */
int gauss(long unsigned *M, int nb_lignes);


/*
 * craque un générateur linéaire "fibonacci" en cherchant les "taps" qui
 * permettent de regénérer la suite.
 * Le tableau 'random' contient les 'nb' premiers bits générés par le
 * générateur.
 * La fonction renvoie 1 si les taps permettent de regénérer la suite, et -1
 * sinon.
 */
int LFSR_crack(int nb, int *random, unsigned long* taps);


/*
 * fonction de test, exécutée par l'argument '-T' de la ligne de commande
 */
void test();

#endif  /* _TP_PRNG_H */


